- [TL;DR](#org366ac75)
- [Topology](#org060c060)
- [Algorithms](#orgc62af2d)
- [Examples](#org0de1012)
- [Batch mode](#orgc90323d)
  - [Running batch simulations with Built-in demons](#org0d65d09)
  - [Running batch simulations with manual demons](#org606c870)
  - [Running batch simulations with `lurette`](#org7545be7)
  - [Viewing Results](#orgb9873d5)
  - [The `sasa` CLI](#org3a2b60e)
- [Interactive mode](#org86bdbf9)
  - [Example: use `rdbg` from the `test/alea-coloring/` directory](#org149bf18)
  - [The exemples of test directory](#org61b0091)
  - [Running interactive sessions with `rdbg`](#org8f622c0)
  - [Getting `rdbg` on-line help](#orgbc10653)
  - [Useful Modules](#orgaacb5c7)
- [Install](#org43df4c0)
  - [Via opam 2 (prefered method)](#org88136d3)
  - [Via docker](#orga982735)
  - [From source](#orge25fa8e)
- [More](#org00d3ccf)
- [FAQ](#orge1ab9f0)
  - [Is there a FAQ?](#orgc6acc57)


# Table of Contents

1.  [TL;DR](#org366ac75)
2.  [Topology](#org060c060)
3.  [Algorithms](#orgc62af2d)
4.  [Examples](#org0de1012)
5.  [Batch mode](#orgc90323d)
    1.  [Running batch simulations with Built-in demons](#org0d65d09)
    2.  [Running batch simulations with manual demons](#org606c870)
    3.  [Running batch simulations with `lurette`](#org7545be7)
    4.  [Viewing Results](#orgb9873d5)
    5.  [The `sasa` CLI](#org3a2b60e)
6.  [Interactive mode](#org86bdbf9)
    1.  [Example: use `rdbg` from the `test/alea-coloring/` directory](#org149bf18)
    2.  [The exemples of test directory](#org61b0091)
    3.  [Running interactive sessions with `rdbg`](#org8f622c0)
    4.  [Getting `rdbg` on-line help](#orgbc10653)
    5.  [Useful Modules](#orgaacb5c7)
7.  [Install](#org43df4c0)
    1.  [Via opam 2 (prefered method)](#org88136d3)
    2.  [Via docker](#orga982735)
    3.  [From source](#orge25fa8e)
8.  [More](#org00d3ccf)
9.  [FAQ](#orge1ab9f0)
    1.  [Is there a FAQ?](#orgc6acc57)


<a id="org366ac75"></a>

# TL;DR

SASA is a **Self-stabilizing Algorithms SimulAtor**, based on the so-called **atomic state model** introduced by <span class="underline">Dijkstra</span> in its seminal article on [Self-stabilizing distributed algorithms](http://www.cs.utexas.edu/~EWD/ewd04xx/EWD426.PDF)

Basically, one needs to provide:

1.  a topology, made of nodes and transitions (via a dot file)
2.  the algorithms attached to nodes (via `ocaml` programs)

The fastest way to get started is to copy the files provides in the `test/skeleton` directory, and to modify them:

```sh
cd test
cp -rf skeleton my_algo
cd my_algo
make ring.cmxs
sasa ring.dot
```

![img](./sasabatch.svg)

The source code is available at <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa>


<a id="org060c060"></a>

# Topology

The topology is given via `.dot` files, that should

1.  follow the [graphviz/dot format](https://en.wikipedia.org/wiki/DOT_(graph_description_language))
2.  have nodes **labelled** by the `algo` field

```dot
graph ring {
 p1 [algo="some_algo.ml"]
 p2 [algo="some_algo.ml"]
 p3 [algo="some_algo.ml"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml"]
 p6 [algo="some_algo.ml"]
 p7 [algo="some_algo.ml"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Of course the `some_algo.ml` file must exist.

In order to define initial state values to nodes, one can use `init` node [attribute](http://www.graphviz.org/doc/info/attrs.html).

```dot
graph ring {
 p1 [algo="some_algo.ml" init="i=1"]
 p2 [algo="some_algo.ml" init="i=2"]
 p3 [algo="some_algo.ml" init="i=42"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml=1"]
 p6 [algo="some_algo.ml=1"]
 p7 [algo="some_algo.ml=1"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

One can use graph [attributes](http://www.graphviz.org/doc/info/attrs.html) to set parameters that can be used by all nodes during the simulation.

```dot
graph ring {
 graph [diameter="4" some_other_global_parameter="42"]
 p1 [algo="some_algo.ml" init="i=1"]
 p2 [algo="some_algo.ml" init="i=2"]
 p3 [algo="some_algo.ml" init="i=42"]
 p4 [algo="some_algo.ml"]
 p5 [algo="some_algo.ml=1"]
 p6 [algo="some_algo.ml=1"]
 p7 [algo="some_algo.ml=1"]

 p1 -- p2 -- p3 -- p4 -- p5 -- p6 -- p7 -- p1 
}
```

Such parameters can be retreived in Algorithms using the `Algo.get_graph_attribute : string -> string` function.

nb: a `Algo.diameter: unit -> int` function is provided, but it can be expensive to use.


<a id="orgc62af2d"></a>

# Algorithms

The following has been generated from [algo.mli](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/blob/master/lib/algo/algo.mli)

<div class="html">
<iframe title="The Algo API" width="700" height="700" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/algo/Algo/index.html>" alt="<sub>html</sub>/algo/Algo/index.html";> </iframe>

</div>


<a id="org0de1012"></a>

# Examples

The [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory contains various examples of self-stabilizing distributed programs taken from the book \`\`Introduction to Distributed Self-Stabilizing Algorithms'' By Altisen, Devismes, Dubois, and Petit.

-   `test/skeleton/`: a fake algorithm meant to be used as a skeleton
-   `test/dijkstra-ring/`: Dijkstra token ring
-   `test/unison/`: Synchronous unison
-   `test/async-unison/`: Asynchronous unison
-   `test/coloring/`: a graph coloring algorithm
-   `test/alea-coloring/`: a randomized variant of the previous one
-   `test/bfs-spanning-tree/`: a Breadth First Search spanning tree construction
-   `test/dfs/`: a Depth First Search using arrays (the \`\`atomic state model'' version of a [Depth First Search algorithm proposed by Collin and Dolev in 1994](http://citeseerx.ist.psu.edu/viewdoc/download?doi=10.1.1.57.1100&rep=rep1&type=pdf))
-   `test/dfs-list/`: a Depth First Search using lists

Each directories contains working exemples, which are checked using the Continuous Integration facilities of Gitlab (cf the [.gitlab-ci.yml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/.gitlab-ci.yml) script and the [CI pipelines](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/pipelines) results).

If you want to reproduce or understand what those non-regression tests do, please look at the `test/*/Makefile`, present in each directory (which all include the [test/Makefile.inc](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/Makefile.inc) one).

The `test` directory also contains sub-directories which gathers programs shared by all examples:

-   `test/lustre/`: contains Lustre programs used as (`test/*/*.lus`) oracles
-   `test/rdbg-utils/`: contains `ocaml` functions that can be used from `rdbg`
-   `test/my-rdbg-tuning.ml`: contains useful shortcuts/commands that can be used from `rdbg`.
-   `test/*/my-rdbg-tuning.ml`: includes `test/my-rdbg-tuning.ml` and defines commands specific to the example of the directory. Indeed, `rdbg`, once launched, first tries to read the content of the file name `my-rdbg-tuning.ml` (it it exists).


<a id="orgc90323d"></a>

# Batch mode


<a id="org0d65d09"></a>

## Running batch simulations with Built-in demons

To launch batch simulations, one can use the `sasa` Command Line Interface.

For instance, in order to simulate an algorithm defined in the `ring.dot` (cf [test/coloring](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/coloring), you can launch the following command:

```sh
sasa ring.dot 
```

By default, the distributed demon will be used to activate actions. Several other demon can be used, via one of the following options:

```sh
sasa -h | grep "\-demon"
```

    --synchronous-demon, -sd
    --central-demon, -cd
    --locally-central-demon, -lcd
    --distributed-demon, -dd
    --custom-demon, -custd


<a id="org606c870"></a>

## Running batch simulations with manual demons

By using the `--custom-demon` option (or `-custd` for short), you can play the role of the demon. More precisely, you will be prompted for stating which actions should be activated, among the set of enabled actions.

```sh
cd ../../test/unison
make ring.cmxs
sasa ring.dot --custom-demon
```

In the custom demon mode, the demon is executed outside `sasa`, by a process that communicates via the standard input/output using [RIF](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-5) conventions. More precisely, `sasa` writes on `stdout` a Boolean for each action and each process, that states if the corresponding action is enabled for the current step. Then it reads on `stdin` a Boolean for each action and each process that tells which actions (among the enabled ones) should be activated.

The demon can thus be played by users that read and enter Boolean values.

In order to enter such input more easily, one can use [`luciole-rif`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-8)

```sh
luciole-rif sasa ring.dot --custom-demon
```

It can also by simulated by [`lutin`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-4) programs via the use of [`lurette`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-9) (for batch executions) or [`rdbg`](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/#outline-container-sec-10) (for interactive sessions).

Built-in demons can of course be programmed in Lutin. One can generate such demons using the `--gen-lutin-demon` option: `sasa --gen-lutin-demon a_graph.dot`. It can be handy at least to get the demons variables names in the good order if one to write its own demon.


<a id="org7545be7"></a>

## Running batch simulations with `lurette`

If you want to perform simulations with custom demons, you need to use `lurette`. For instance, in order to simulate an algorithm defined in the `ring.dot` (cf [test/coloring](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/coloring)) using a Lutin demon defined in `ring.lut`, you can launch the following command:

```sh
lurette \
     -env "sasa ring.dot -custd" \
     -sut "lutin ring.lut -n distributed"
```

For `lurette`, the role of the SUT and the one of the environment is dual: the outputs of the SUT are the inputs of the environment, and vice-versa. The only difference is that the environment plays first. But `sasa` needs to play first, to be able to state which actions are enabled at the very first step. Hence `sasa` is used as a `lurette` environment, and the daemon program is used a `lurette` SUT.

If one wants to use an internal sasa daemon, no environment is needed and sasa can be used with `-sut`. In order to use an oracle defined in Lustre, one can use `lurette` as follows:

```sh
lurette \
     -sut "sasa ring.dot" \
     -oracle "lv6 ring_oracle.lus -n oracle -exec"
```


<a id="orgb9873d5"></a>

## Viewing Results

`sasa -rif` and `lurette` generates `.rif` files that can be viewed with `gnuplot-rif` or `sim2chro`; cf <http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6/>


<a id="org3a2b60e"></a>

## The `sasa` CLI

```sh
sasa --help
```

    usage: sasa [<option>]* <topology>.dot 
    use -h to see the available options.
    
    --synchronous-demon, -sd
                Use a Synchronous demon
    --central-demon, -cd
                Use a Central demon (selects exactly one action)
    --locally-central-demon, -lcd
                Use a Locally Central demon
                (i.e., never activates two neighbors actions in the same step)
    --distributed-demon, -dd
                Use a Distributed demon (which select at least one action). This is the default demon.
    --custom-demon, -custd
                Use a Custom demon (forces --rif)
    --seed, -seed
                Set the pseudo-random generator seed of build-in demons (wins over --replay)
    --replay, -replay
                Use the last generated seed to replay the last run
    --gen-register, -reg
                Generates the registering file and exit. 
    --length, -l <int>
                Maximum number of steps to be done (10000 by default).
    
    --version, -version, -v
                Display the sasa version and exit.
    --verbose, -vl <int>
                Set the verbose level
    --help, -help, -h
                Display main options
    --more, -m  Display more options

More `sasa` options:

```sh
sasa --more
```

    --rif, -rif Display only outputs on stdout (i.e., behave as a rif input file)
    --no-data-file, -nd
                Do not generate any data file
    --gen-lutin-demon, -gld
                Generate Lutin demons and exit
    --gen-lustre-oracle-skeleton, -glos
                Generate a Lustre oracle skeleton
    --list-algos, -algo
                Output the algo files used in the dot file and exit. 
    --dummy-input
                Add a dummy input to sasa so that built-in demon can be used from rdbg
    --ignore-first-inputs, -ifi
                Ignore first inputs (necessary to use luciole via 
                lurette/rdbg/luciole-rif)
    --ocaml-version
                Display the version ocaml version sasa was compiled with and exit.


<a id="org86bdbf9"></a>

# Interactive mode

If you want to perform interactive session, you can launch `sasa` under the control of [rdbg](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/rdbg/). Before reading this section, please read at least the [Basic usage Section if the rdbg documentation](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/rdbg/README.html#Level0).


<a id="org149bf18"></a>

## Example: use `rdbg` from the `test/alea-coloring/` directory

All the sub-directories of the `test` directory are organized similarly. Let's have a look at the `test/alea-coloring/` directory. It contains the following files:

-   `my-rdbg-tuning.ml` that just includes [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) (some `my-rdbg-tuning.ml` files define more commands, such as the one in [test/async-unison/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/async-unison/my-rdbg-tuning.ml)).
-   `ring.dot`: a possible topology
-   `p.ml`: an algorithm used in `ring.dot`
-   `Makefile`: a Makefile that demonstrates various ways of using `sasa`. In particular, it contains a rule named `rdbg`.

Try for instance:

```sh
cd test/coloring
make rdbg
```

This make rule (defined in `Makefile` and `../Makefile.inc`)

1.  generates the `ring.ml` file, that contains the registration code (that can indeed be generated by sasa, as explained in `algo.mli`).
2.  launches `rdbg` with some arguments. `rdbg` then prompts the user to enter one of the folling commands:

```sh
rdbg -o ring.rif -sut "$(sasa)  ring.dot --locally-central-demon" 

[] create a fresh session
[q] quit
[s] do nothing

[/q/s]: 
```

By default, the first in the list is executed; hence if you press `[Return]` without any command, a fresh session is created. More precisely it creates a `rdbg_session.ml` file, that loads all the necessary modules to run an interactive session with `rdbg`, where `rdbg` simulates ring.dot with `sasa` using a locally distributed daemon.

For instance, you can type `nr[return]`, a `rdbg` command defined in [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) that steps until the next round is reached. Then, by typing `[return]`; `rdbg` will run the lastly used command, which is `nr`, which allow you to move forward in the execution from rounds to rounds. If you want to go backwards to the previous round, you can use the `br` command=, also defined in [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml).

nb: the next time `rdbg` is launched (without argument), you will be prompted with the possibility to reuse this session.

```sh
$ rdbg

[] #use "rdbg-session.ml"
[c] create a fresh session
[q] quit
[s] do nothing

[/c/q/s]: 
```

Typing `[Enter]` will therefore also load the `rdbg_session.ml` file we have just been using.


<a id="org61b0091"></a>

## The exemples of test directory

The [test](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test) directory contains several files and directories that take advantage of the versatility (programmability) of `rdbg` to perform interactive `sasa` simulations:

-   `test/rdbg-utils/`: contains `ocaml` functions that can be used from `rdbg`
-   `test/my-rdbg-tuning.ml`: contains useful shortcuts/commands that can be used from `rdbg`.
-   `test/*/my-rdbg-tuning.ml`: includes `test/my-rdbg-tuning.ml` and defines commands specific to the example of the directory. Indeed, `rdbg`, once launched, first tries to read the content of the file name `my-rdbg-tuning.ml` (it it exists).


<a id="org8f622c0"></a>

## Running interactive sessions with `rdbg`

1.  type `rdbg`
2.  press enter to load the defaut session (`rdbg-session.ml`). Then you can type:
3.  `d` for displaying a (dynamic) dot graph with `dot`
4.  or ALTERNATIVELY `ne` to use the `neato` layout engine (`tw`, `ci`, `fd`, `sf`, `pa`, `os` to use the `twopi`, `circo`, `fdp`, `sfd`, `patchwork`, `osage` layout engines respectively)
5.  `s` to move one step forward
6.  `sd` to move one step forward and call `d` (to update the graph display [])
7.  `nr` to move one round forward
8.  `pr` to move one round backward
9.  `go` to move forward until convergence

All those commands are defined in (the common) [test/my-rdbg-tuning.ml](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/test/my-rdbg-tuning.ml) that is included in (locals) `test/*/my-rdbg-tuning.ml` that are included in (generated) `test/*/rdbg-session.ml` files. `my-rdbg-tuning.ml` contains straigthforward `ocaml` code that defines various `rdbg` shortcuts to ease the simulation of `sasa` systems. Feel free to tailor those command to yours needs by modyfying the local `my-rdbg-tuning.ml`!


<a id="orgbc10653"></a>

## Getting `rdbg` on-line help

Here are 2 useful entry-points to rdbg on-line help:

1.  `rdbg --help` and
2.  at the `rdbg` prompt, you can use the `l` command:

```sh
(rdbg) l
```

    8d65906ad6023baebe09c6f8684ab537


<a id="orgaacb5c7"></a>

## Useful Modules

Some modules, used by the sasa core engine, can be useful from `rdbg`.

<div class="html">
<iframe title="The Algo API" width="700" height="500" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Topology/index.html>" alt="<./_html/sasacore/Sasacore/Topology/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The Algo API" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Diameter/index.html>" alt="<./_html/sasacore/Sasacore/Diameter/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The Algo API" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/Process/index.html>" alt="<./_html/sasacore/Sasacore/Process/index.html>";> </iframe>

</div>

<div class="html">
<iframe title="The Algo API" width="700" height="300" src="<https://verimag.gricad-pages.univ-grenoble-alpes.fr/synchrone/sasa/_html/sasacore/Sasacore/StringOf/index.html>" alt="<./_html/sasacore/Sasacore/StringOf/index.html>";> </iframe>

</div>


<a id="org43df4c0"></a>

# Install


<a id="org88136d3"></a>

## Via opam 2 (prefered method)

```sh
opam repo add verimag-sync-repo "http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/opam-repository"
opam update -y
opam install -y sasa
```

Once this is done, upgrading to the last version of the tools is as simple as:

```yaml
opam update
opam upgrade
```

In order to install the tools mentionned in this documentation that can be used with `sasa` (namely, `gnuplot-rif`, `sim2chro`, `luciole-rif`, `lutin`, `lurette`, `rdbg`):

```sh
opam depext -y rdbg lutin
opam install -y rdbg lutin
```


<a id="orga982735"></a>

## Via docker

cf the Install section of [Synchrone Reactive Tool Box](http://www-verimag.imag.fr/DIST-TOOLS/SYNCHRONE/lustre-v6).


<a id="orge25fa8e"></a>

## From source

You will need:

-   `make` (gnu)
-   a set of tools installable via [opam](https://opam.ocaml.org/)
    -   `dune`
    -   `ocamlgraph`
    -   `rdbg`
    -   `lutin` (not for compiling actually, but for using sasa with custom demons)

One can mimick the content of the `test` job in the project [.gitlab-ci.yml Gitlab CI script](https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/tree/master/.gitlab-ci.yml).


<a id="org00d3ccf"></a>

# More

-   Releases Notes: <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa/releases>
-   Sources: <https://gricad-gitlab.univ-grenoble-alpes.fr/verimag/synchrone/sasa>


<a id="orge1ab9f0"></a>

# FAQ


<a id="orgc6acc57"></a>

## Is there a FAQ?

yes
