(* Time-stamp: <modified the 05/07/2019 (at 17:29) by Erwan Jahier> *)

open Process

let b2s b = if b then "t" else "f"

let (array_to_string : bool array -> string) =
  fun a ->
    let l = Array.fold_right (fun b acc -> (b2s b)::acc) a [] in
    "["^(String.concat "," l)^"]"

let (matrix_to_string : bool array array -> string) =
  fun m -> 
    let l = Array.fold_right (fun a acc -> (array_to_string a)::acc) m [] in
    "[\n\t"^(String.concat ",\n\t" l)^"]"

let graph_attributes_to_string () =
  let al = Register.graph_attribute_list () in
  let l = List.map (fun (n,v) -> Printf.sprintf "const %s=%s;\n" n v) al in
  String.concat "" l
  
let (f: Topology.t -> 'v Process.t list -> string) =
  fun g pl ->
    let actions_nb = List.map (fun p -> List.length p.actions) pl in
    let m = List.fold_left max (List.hd actions_nb) (List.tl actions_nb) in
    let n = List.length pl in

    let al = List.map (fun p -> List.map (fun a -> p.pid^"_"^a) p.actions) pl in
    let al = List.flatten al in
    let enabl = List.map (fun a -> "Enab_"^a) al in
    let input_decl = (String.concat ","(al@enabl)) ^ ":bool" in
    let array_decl = Printf.sprintf "\tActi:bool^%d^%d;\n\tEnab:bool^%d^%d;\n" m n m n in
    let acti_name p a = Printf.sprintf "%s_%s" p.pid a in
    let enab_name p a = Printf.sprintf "Enab_%s_%s" p.pid a in
    let array_def_acti =
      Printf.sprintf "\tActi = [%s];\n"
        (String.concat ","
           (List.map (fun p ->
                "["^(String.concat "," (List.map (acti_name p) p.actions))^"]") pl))
    in
    let array_def_enab =
      Printf.sprintf "\tEnab = [%s];\n"
        (String.concat ","
           (List.map (fun p ->
                "["^(String.concat "," (List.map (enab_name p) p.actions))^"]") pl))
    in
    let algo = Filename.basename (Sys.getcwd()) in
    let algo = (* Make sure that we generate valid Lustre idents *)
      Str.global_replace (Str.regexp "[-,\\.]") "_" algo
    in
    Printf.sprintf "%sinclude \"%s_oracle.lus\"
const an=%d; -- actions number
const pn=%d; -- processes number
const degree=%d;
const min_degree=%d;
const mean_degree=%f;
const diameter=%d;
const card=%d;
const links_number=%d;
const is_cyclic=%b;
const is_connected=%b;
const is_a_tree=%b;
const adjency=%s;
%s
node oracle(%s) returns (ok:bool);
var
  %slet
  %s  %s
  ok = %s_oracle<<an,pn>>(Enab,Acti);
tel
"
      (Mypervasives.entete "--"  SasaVersion.str SasaVersion.sha)
      algo
      m n
      (Register.max_degree ())
      (Register.min_degree ())
      (Register.mean_degree ())
      (Register.diameter ())
      (Register.card ())
      (Register.links_number ())
      (Register.is_cyclic ())
      (Register.is_connected ())
      (Register.is_tree ())
      (matrix_to_string (Topology.to_adjency g))
      (graph_attributes_to_string ())
      input_decl
      array_decl
      array_def_acti
      array_def_enab
      algo


