(* Time-stamp: <modified the 17/10/2019 (at 21:16) by Erwan Jahier> *)

(**  This module  duplicates and  extends the  Algo module  with get_*
   functions.

    The rationale for defining such a module is to be able to hide the
   get_*  functions from  the Algo  interface. Indeed,  they are  only
   called  by the  sasa engine,  and it  is dangerous  and useless  to
   expose them to algorithms designers.  *)

type 's neighbor = {
  state:  's ;
  pid: unit -> string; 
  spid: unit -> string; 
  reply: unit -> int; 
  weight: unit -> int option; 
}

type algo_id = string
type action  = string 
type 's enable_fun = 's neighbor list -> 's -> action list
type 's step_fun   = 's neighbor list -> 's -> action -> 's


val reg_init_state : algo_id -> (int -> 's) -> unit
val reg_enable : algo_id -> 's enable_fun -> unit
val reg_step : algo_id -> 's step_fun -> unit
val reg_actions : algo_id -> action list -> unit
val reg_value_to_string : ('s -> string) -> unit
val reg_value_of_string : (string -> 's) -> unit
val reg_copy_value : ('s -> 's) -> unit

val card : unit -> int
val min_degree : unit -> int
val mean_degree : unit -> float
val max_degree : unit -> int
val is_cyclic : unit -> bool
val is_connected : unit -> bool
val is_tree : unit -> bool
val is_directed : unit -> bool

(** If height () = None, then the graph doesn't have a height (because it isn't a tree)
   Otherwise, height () = Some h.*)
val height : unit -> (string -> int) option
val links_number : unit -> int
val diameter : unit -> int

val verbose_level: int ref

val get_graph_attribute : string -> string


(* All the remaining functions are used only by the sasa Engine, while
   the functions above are only used in the Algo module (i.e., by
   distributed algorithm designers. *)

val get_enable : algo_id -> 's enable_fun
val get_step   : algo_id -> 's step_fun
val get_init_state : algo_id -> int -> 's
val get_actions   : algo_id -> action list
val get_value_to_string : unit -> 's -> string
val get_value_of_string : unit -> (string -> 's) option
val get_copy_value : unit -> ('s -> 's)
val to_string : 's -> string


val set_card : (unit -> int) -> unit
val set_degrees : (unit -> int*int) -> unit
val set_mean_deg : (unit -> float) -> unit
val set_is_connected_cyclic : (unit -> bool*bool) -> unit
val set_is_directed : (unit -> bool) -> unit
type node_id = string (* cf topology.mli *)
val set_height : (node_id -> int) -> unit
val set_links_number : (unit -> int) -> unit
val set_diameter : (unit -> int) -> unit

val set_graph_attribute : string -> string -> unit
val graph_attribute_list: unit -> (string * string) list

