open Sasacore

let (print_step : int -> int -> SasArg.t -> 'v Env.t -> 'v Process.t list -> string ->
                  bool list list -> unit) =
  fun n i args e pl activate_val enab_ll ->
  let enable_val =
    String.concat " " (List.map (fun b -> if b then "t" else "f")
                         (List.flatten enab_ll))
  in
  if args.no_data_file then (
    Printf.printf "\n#step %s\n%!" (string_of_int (n-i+1))
  ) else (
    if args.demon = Demon.Custom then (
      (* in custom mode, to be able to talk with lurette, this should not be 
           printed on stdout
       *)
      Printf.eprintf "\n#step %s\n" (string_of_int (n-i+1)) ;
      Printf.eprintf "%s #outs " activate_val; flush stderr;
      Printf.printf "%s %s\n" (StringOf.env_rif e pl) enable_val;
    ) else (
      (* rif mode, internal demons *)
      if args.rif then
        Printf.printf " %s %s %s\n" (StringOf.env_rif e pl) enable_val activate_val
      else (
        Printf.printf "\n#step %s\n" (string_of_int (n-i+1));
        Printf.printf "#outs %s %s %s\n" (StringOf.env_rif e pl) enable_val activate_val
      );
    );
    flush stderr;
    flush stdout
  )
  
exception Silent of int

let (simustep: int -> int -> SasArg.t -> string -> 
      ('v Process.t * 'v Register.neighbor list) list -> 'v Env.t -> 'v Env.t * string) =
  fun n i args activate_val pl_n e ->
    (* 1: Get enable processes *)
    let all, enab_ll = Sasacore.Main.get_enable_processes pl_n e in
    let pl = fst(List.split pl_n) in
    if
      not (args.rif) && List.for_all (fun b -> not b) (List.flatten enab_ll)
    then (
      print_step n i args e pl activate_val enab_ll;
      raise (Silent (n-i+1))
    ) else 
    if args.demon = Demon.Custom then
      print_step n i args e pl activate_val enab_ll;
    (* 2: read the actions *)
    let get_action_value = RifRead.bool (args.verbose > 1) in
    let next_activate_val, pnal = Demon.f args.dummy_input
        (args.verbose > 1) args.demon pl all enab_ll get_action_value 
    in
    let next_activate_val =
      String.concat " " 
        (List.map
           (fun b -> if b then "t" else "f")
           (List.flatten next_activate_val)
        )
    in
    (* 3: Do the steps *)
    let ne = Sasacore.Main.do_step pnal e in
    if args.demon <> Demon.Custom then
      print_step n i args e pl next_activate_val enab_ll;
    ne, next_activate_val


let rec (simuloop: int -> int -> SasArg.t -> string -> 
         ('v Process.t * 'v Register.neighbor list) list -> 'v Env.t -> unit) =
  fun n i args activate_val pl_n e ->

    let ne, next_activate_val = simustep n i args activate_val pl_n e in
    if i > 0 then simuloop n (i-1) args next_activate_val pl_n ne else (
      print_string "#q\n"; flush stdout
)

let () =
  let args, pl_n, e = Sasacore.Main.make true Sys.argv in
  try
    let n = args.length in
    simuloop n n args "" pl_n e
  with
  | Failure msg -> Printf.printf "Error: %s\n" msg
  | Silent i ->
    let str = if args.rif then "#" else "" in
    Printf.eprintf "\n%sThis algo is silent after %i steps\n" str i ;
    flush stderr;
    flush stdout;
    print_string "\n#quit\n"; 
    flush stderr;
    flush stdout
