(* Time-stamp: <modified the 14/10/2019 (at 22:47) by Erwan Jahier> *)

(* cf Collin-Dolex-94 *)

open Algo
open State

let actions = Some ["update_path";"compute_parent"]


let (init_state: int -> State.t ) =
  fun i ->
    {
      path = Array.make delta (-1);
      par  = try Random.int i with _ -> assert false
    }

let str_of_array a =
  let l = List.map State.to_string (Array.to_list a) in
  "["^(String.concat "," l)^"]"

let min_path al =
      match al with
      | [] -> assert false
      | x::t -> List.fold_left min x t

let (get_paths: 'v neighbor list -> int array list) =
  fun nl ->
    List.mapi (fun alpha_i n -> (state n).path) nl 

(* The index of the first negative elt in a starting from 1. Returns
   the size of the a if none is found *)
let end_of_a a =
  let rec aux a i =
    if i=delta then delta else
    if a.(i) < 0 then i else aux a (i+1) 
  in
  let res = aux a 1 in
  res
    
(* concat or truncate *)
let (concat_path : int array -> int -> int array) =
  fun a alpha ->
    let a = Array.copy a in (* ouch! Bad things happen without this copy *)  
    let s = Array.length a in
    let last = end_of_a a in
    if last = s then ( (* truncate *)
      for i = 1 to s-1 do
        a.(i-1) <- a.(i)
      done;
      a.(s-1) <- alpha
    )
    else (* concat *)
      a.(last) <- alpha;
    a

let array_to_string a = String.concat ";" (List.map string_of_int (Array.to_list a))

let equals_up_to p1 p2 i =
  let res = ref true in
  for j = 0 to i do
    res := !res && p1.(j) = p2.(j) ;
  done;
  !res

let (compute_parent : t neighbor list -> int array -> int) =
  fun nl p ->
    (* The parent of a process p is the neighbor which path is a
       subpath of p *)
    let last_p = end_of_a p in
    let paths = List.mapi (fun i n -> (state n).path, i) nl in
    let _,parent =
      match 
        List.find_opt
          (fun (npath, _) ->
             let last_n = end_of_a npath in             
             last_n = last_p-1 && equals_up_to npath p (last_n-1)
          )
          paths
      with
        Some p -> p
      | None -> [||],(-1) (* may happen before convergence *)
    in
    parent

let compute_path nl e =
  let paths = get_paths nl in
  let paths = List.map2 (fun p n -> concat_path p (reply n)) paths nl in
  let path = min_path paths in
  path
    
let (enable_f: t -> t neighbor list -> action list) =
  fun v nl ->
    let path = compute_path nl v in
    if path <> v.path then ["update_path"] else
    if compute_parent nl path <> v.par then ["compute_parent"]
    else []

let (step_f : t -> t neighbor list -> action -> t) =
  fun v nl -> 
    function
    | "update_path" ->
      let path = compute_path nl v in
      { v with path = path }
    | "compute_parent" ->
      let path = compute_path nl v in
      { v with par = compute_parent nl path }
    | _ -> assert false


